import { Tr } from "pi_ucore/db/mgr";
import { getCfg, getAllCfg, writes } from "pi_common_tools/util/db_tool";
import { costProps, sendAwards , getCounts } from 'pi_award/api';
import { read, write } from "pi_common_tools/util/db_tool";
import { throwEvent } from "pi_throw_event/api";
import { getHeroDbByModeID } from "game_hero_info_server/api";
import { LingBaoInstance, PlayerLingBaoDB, StarUpConsume, OperationError } from "game_special_equip/struct/LingBaoSystem.struct";

import { LingBaoCfg, getStarGroupId } from "game_special_equip/xls/lingbao/lingbao_base.struct";
import { StarCfg } from "game_special_equip/xls/lingbao/lingbao_star.struct";
import { LingBaoChip } from "game_special_equip/xls/lingbao/lingbao_fragment.struct";

import { PartnerBase } from "game_monster_base/xls/partner/partner_base.struct";
import { changeProp } from 'game_bag_mgr/api';



//-----------------------------------------------------------------------
// 灵宝系统API接口
//-----------------------------------------------------------------------
/**
 * @name 执行灵宝解锁操作
 * @param tr 事务对象
 * @param rid 玩家唯一rid
 * @param instanceId 灵宝实例ID
 */
export const unlockLingBao = async (tr: Tr, rid: number, instanceId: number) => {
    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;
    if (db.instances.has(instanceId)) return { r: false, msg: "已经解锁灵宝" };       //已经解锁了灵宝了
    let baseCfgInfo = getCfg(LingBaoCfg, instanceId);
    if (!baseCfgInfo) return { r: false, msg: "参数错误" };

    let instance = new LingBaoInstance();
    instance.special_instance_id = instanceId;
    instance.star_level = 0;
    instance.hero_id = 0;
    db.instances.set(instance.special_instance_id, instance);
    // await write(tr, PlayerLingBaoDB, rid, db);

    return { r: true, arr: [PlayerLingBaoDB, rid, db] };
}


/**
 * @name 执行灵宝升星操作
 * @param tr 事务对象
 * @param rid 玩家唯一rid
 * @param instanceId 灵宝实例ID
 */
export const starUpLingBao = async (tr: Tr, rid: number, instanceId: number) => {
    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;
    if (!db.instances.has(instanceId)) {
        return { r: false, msg: "未解锁灵宝" }     //未解锁灵宝
    }
    let instance = db.instances.get(instanceId);

    let baseCfgInfo = getCfg(LingBaoCfg, instanceId);
    if (!baseCfgInfo) return { r: false, msg: "参数错误" };

    let groupId = getStarGroupId(baseCfgInfo.starGroup, instance.star_level);
    let starCfgInfo = getCfg(StarCfg, groupId);
    if (!starCfgInfo) return { r: false, msg: "配置错误" };


    if (!starCfgInfo.upCost || !starCfgInfo.upCost.length) {
        return { r: false, msg: "星级已达满级" }
    }

    // 扣除消耗材料
    const costResult = await costProps(tr, rid, starCfgInfo.upCost);
    if (!costResult) {
        return { r: false, msg: "材料不足" }
    }
    // 执行升星
    instance.star_level += 1;
    await write(tr, PlayerLingBaoDB, rid, db);

    // 通知战力变化
    if (instance.hero_id) {
        await notifyPowerChange(tr, rid, instance.hero_id);
    }

    return instance;
}


/**
 * @name 装备灵宝到英雄
 * @param tr 事务对象
 * @param rid 玩家唯一rid
 * @param heroId 英雄ID
 * @param instanceId 灵宝实例ID
 */
export const equipToHero = async (tr: Tr, rid: number, heroId: number, instanceId: number) => {
    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;
    if (!db.instances.has(instanceId)) {
        return { r: false, msg: "未解锁灵宝" }     //未解锁灵宝
    }

    let oldLinbaoId = db.hero_equips.get(heroId);
    if (oldLinbaoId) {
        db.instances.get(oldLinbaoId).hero_id = 0;
    }

    // 检查灵宝是否已装备
    let lingbao = db.instances.get(instanceId);

    if (lingbao.hero_id) {
        // return {r : false,msg : "灵宝已经装备"}     //未解锁灵宝
        db.hero_equips.delete(lingbao.hero_id);        //删除当前灵宝匹配的卡牌
    }

    //绑定到当前卡牌
    lingbao.hero_id = heroId;
    db.hero_equips.set(lingbao.hero_id, instanceId);

    await write(tr, PlayerLingBaoDB, rid, db);

    // 通知战力变化
    await notifyPowerChange(tr, rid, heroId);

    return lingbao;
}

/**
 * @name 卸下灵宝
 * @param tr 事务对象
 * @param rid 玩家唯一rid
 * @param heroId 英雄ID
 */
export const unequipFromHero = async (tr: Tr, rid: number, heroId: number) => {
    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;

    if (!db.hero_equips.has(heroId)) {
        return { r: false, msg: "未装备灵宝" }     //未装备灵宝
    }

    // 
    let instanceId = db.hero_equips.get(heroId);
    // 检查灵宝
    let lingbao = db.instances.get(instanceId);

    if (lingbao.hero_id !== heroId) {
        return { r: false, msg: "数据错误" }     //未装备灵宝
    }

    lingbao.hero_id = 0;
    db.hero_equips.delete(heroId);

    await write(tr, PlayerLingBaoDB, rid, db);

    // 通知战力变化
    await notifyPowerChange(tr, rid, heroId);

    return lingbao;
}

export const getEquipNum = async (tr: Tr, rid: number) => {
    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;
    let lingBaoHasList = [];
    let lingBaoNotHasList = [];
    let lingBaoCfg = getAllCfg(LingBaoCfg);
    lingBaoCfg.forEach((v, key) => {
        let hasLingBao = db.instances.get(key);
        if (hasLingBao) {
            lingBaoHasList.push(hasLingBao);
        } else {
            lingBaoNotHasList.push(v)
        }
    })
    return { lingBaoHasList, lingBaoNotHasList };
}

export const getEquipStar = async (tr: Tr, rid: number, lingBaoId: number) => {
    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;
    // 检查是否已解锁
    let lingBao = db.instances.get(lingBaoId);
    if (!lingBao) return -1;
    return lingBao.star_level;
}


function hasDuplicateElements(arr: any[][]) {
    const seen = new Set();
    for (const subArr of arr) {
        for (const element of subArr) {
            if (seen.has(element)) {
                return { result : false , set : seen};
            }
            seen.add(element);
        }
    }
    return {result : true, set : seen};
}



/**
 * @name 互换灵宝
 * @param tr 事务对象
 * @param rid 玩家唯一rid
 * @param param : {change : [[heroIdA,heroIdB]]}
 */
export const exchangeLingBao = async (tr: Tr, rid: number, param: any) => {

    const db = await read(tr, PlayerLingBaoDB, rid) as PlayerLingBaoDB;

    let r = hasDuplicateElements(param.change);
    if(!r.result)return {r : false,msg : "参数重复"};

    for (const hero_id of r.set) {
        if(db.hero_equips.has(hero_id)){
            let instanceId = db.hero_equips.get(hero_id);   // 灵宝id
            db.hero_equips.delete(hero_id)
            let lingbao = db.instances.get(instanceId);
            if(!lingbao)return {r : false,msg : "灵宝未初始化，存在异常:"+instanceId};
            lingbao.hero_id = 0;        //灵宝全部卸下
            db.instances.set(instanceId,lingbao);
        }
    }

    for(let arr of param.change){
        let heroIdA = arr[0],heroIdB = arr[1];
        let heroCfgInfoA = getCfg(PartnerBase,heroIdA);
        let heroCfgInfoB = getCfg(PartnerBase,heroIdB);
        
        let lingbaoA = db.instances.get(heroCfgInfoA.exclusive_equip);
        let lingbaoB = db.instances.get(heroCfgInfoB.exclusive_equip);
        
        if(lingbaoA && !lingbaoB){
            lingbaoB = new LingBaoInstance();
            lingbaoB.special_instance_id = heroCfgInfoB.exclusive_equip;
            lingbaoB.star_level = lingbaoA.star_level;
            lingbaoB.hero_id = 0;

            db.instances.delete(heroCfgInfoA.exclusive_equip);
        }

        if(!lingbaoA && lingbaoB){
            lingbaoA = new LingBaoInstance();
            lingbaoA.special_instance_id = heroCfgInfoA.exclusive_equip;
            lingbaoA.star_level = lingbaoB.star_level;
            lingbaoA.hero_id = 0;

            db.instances.delete(heroCfgInfoB.exclusive_equip);
        }

        if(lingbaoA && lingbaoB){
            //星级交换
            let tempLevel = lingbaoA.star_level;
            lingbaoA.star_level = lingbaoB.star_level;
            lingbaoB.star_level = tempLevel;
            
            db.instances.set(lingbaoA.special_instance_id,lingbaoA);
            db.instances.set(lingbaoB.special_instance_id,lingbaoB);
        }

        let baseCfgInfoA = getCfg(LingBaoCfg,heroCfgInfoA.exclusive_equip);
        let baseCfgInfoB = getCfg(LingBaoCfg,heroCfgInfoB.exclusive_equip);

        
        //处理碎片
        let chipA = baseCfgInfoA.chipId;
        let chipB = baseCfgInfoB.chipId;

        await changeProp(tr,rid,[chipA,chipB]);
    }

    await write(tr, PlayerLingBaoDB, rid, db);
    
    return {r : true,msg : "ok"};
}


const notifyPowerChange = async (tr: Tr, rid: number, heroId: number) => {
    const heroDb = await getHeroDbByModeID(tr, rid, heroId);
    if (heroDb) {
        console.log("notifyPowerChange==========>", heroId, heroDb.heroId);
        await throwEvent(tr, rid, "specialEquip/api/lingbaoPowerChange", heroDb.heroId);
    }else {
        console.error(`英雄配置id: ${heroId} 找不到对应的英雄唯一id !!`);
    }
}

